// Naive algorithm searching almost all nodes
function findCaretNode(node, row, column) {
  let maxNode = null;
  let minNode = null;
  // Check to see if we found the element
  if(node && node instanceof HTMLElement && node.hasAttribute("data-sourcepos")) {
    // Example string: "26:1-26:4"
    let str = node.getAttribute("data-sourcepos");
    let rows = str.split("-");
    let [row1,column1] = rows[0].split(":");
    let [row2,column2] = rows[1].split(":");
    if(row >= parseInt(row1)) {
      maxNode = node;
    }
    if(row <= parseInt(row2)) {
      minNode = node;
    }

    // In this case, no need to recurse?
    if(!maxNode && !minNode) {
      return null;
    }
  }

  // Recurse
  let nodes = node.childNodes;
  for (let i = 0; i <nodes.length; i++){
    // Optimize: Do a binary search of the nodes based on the row value.
    let maxMinNodes = findCaretNode(nodes[i], row, column);
    if(maxMinNodes) {

      let childMaxNode = maxMinNodes[0];
      let childMinNode = maxMinNodes[1];
      if(childMaxNode) {
        maxNode = childMaxNode;
      }

      if(childMinNode && (!minNode || childMaxNode)) {
        minNode = childMinNode;
      }
    }
  }

  return [maxNode, minNode];
}

function updateHeight() {
  document.documentElement.style.setProperty('height', 'auto', 'important');
  document.documentElement.style.setProperty('minHeight', 'auto', 'important');

  let caretHeight = -1.0;
  let row = parseInt(document.body.getAttribute("selected_pos"));
  if(!isNaN(row) && row != -1) {
    let fraction = parseFloat(document.body.getAttribute("fraction"));
    let column = 0;

    let [maxNode, minNode] = findCaretNode(document.documentElement, row, column);
    if(maxNode) {
      if(!minNode) {
        minNode = maxNode;
      }

      if(maxNode == minNode && fraction == -1.0) {
        let str = maxNode.getAttribute("data-sourcepos");
        let rows = str.split("-");
        let [row1,column1] = rows[0].split(":");
        let [row2,column2] = rows[1].split(":");
        let nom = row-parseInt(row1)+1-0.5; // Subtracting 0.5 to center line
        let denom = parseInt(row2)-parseInt(row1)+1;

        caretHeight = maxNode.getBoundingClientRect().top + nom*maxNode.getBoundingClientRect().height / denom;
      }
      else {
        if(fraction == -1.0) {
          fraction = 0.5;
        }
        caretHeight = maxNode.getBoundingClientRect().bottom + (1.0-fraction)*(maxNode.getBoundingClientRect().bottom - minNode.getBoundingClientRect().top);
      }
    }
  }

  let height = document.documentElement.getBoundingClientRect().height;
  return [height, caretHeight];
}


// Old code looking for "caretLocation". We could return to something like this if we preprocessed the HTML adding a value to the wanted data-sourcepos element.
// function updateHeight(locate_caret) {
// 	document.documentElement.style.setProperty('height', 'auto', 'important');
// 	document.documentElement.style.setProperty('minHeight', 'auto', 'important');
// 	var caretElement = document.getElementById('caretLocation');
//
// 	var height = document.documentElement.getBoundingClientRect().height;
// 	var caretHeight = caretElement ? caretElement.getBoundingClientRect().top : -1.0;
// 	return [height, caretHeight];
// }
